@@ -1,3 +1,24 @@
+/*
+ * Clutter.
+ *
+ * An OpenGL based 'interactive canvas' library.
+ *
+ * Copyright (C) 2009,2010  Intel Corporation.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ */
+
 /**
  * SECTION:clutter-box
  * @short_description: A Generic layout container
@@ -118,10 +139,6 @@ clutter_box_real_add (ClutterContainer *container,
 
   clutter_actor_set_parent (actor, CLUTTER_ACTOR (container));
 
-  clutter_layout_manager_add_child_meta (priv->manager,
-                                         container,
-                                         actor);
-
   clutter_actor_queue_relayout (actor);
 
   g_signal_emit_by_name (container, "actor-added", actor);
@@ -140,10 +157,6 @@ clutter_box_real_remove (ClutterContainer *container,
   priv->children = g_list_remove (priv->children, actor);
   clutter_actor_unparent (actor);
 
-  clutter_layout_manager_remove_child_meta (priv->manager,
-                                            container,
-                                            actor);
-
   clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 
   g_signal_emit_by_name (container, "actor-removed", actor);
@@ -157,10 +170,12 @@ clutter_box_real_foreach (ClutterContainer *container,
                           gpointer          user_data)
 {
   ClutterBoxPrivate *priv = CLUTTER_BOX (container)->priv;
-  GList *l;
 
-  for (l = priv->children; l != NULL; l = l->next)
-    (* callback) (l->data, user_data);
+  /* Using g_list_foreach instead of iterating the list manually
+     because it has better protection against the current node being
+     removed. This will happen for example if someone calls
+     clutter_container_foreach(container, clutter_actor_destroy) */
+  g_list_foreach (priv->children, (GFunc) callback, user_data);
 }
 
 static void
@@ -181,7 +196,7 @@ clutter_box_real_raise (ClutterContainer *container,
       priv->children = g_list_insert (priv->children, actor, index_);
     }
 
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -202,7 +217,7 @@ clutter_box_real_lower (ClutterContainer *container,
       priv->children = g_list_insert (priv->children, actor, index_);
     }
 
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -212,7 +227,7 @@ clutter_box_real_sort_depth_order (ClutterContainer *container)
 
   priv->children = g_list_sort (priv->children, sort_by_depth);
 
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -277,7 +292,7 @@ clutter_box_real_get_preferred_width (ClutterActor *actor,
   /* if we don't have any children don't bother proxying the
    * call to the layout manager instance
    */
-  if (priv->children == NULL)
+  if (priv->children == NULL || priv->manager == NULL)
     {
       if (min_width)
         *min_width = 0.0;
@@ -305,7 +320,7 @@ clutter_box_real_get_preferred_height (ClutterActor *actor,
   /* if we don't have any children don't bother proxying the
    * call to the layout manager instance
    */
-  if (priv->children == NULL)
+  if (priv->children == NULL || priv->manager == NULL)
     {
       if (min_height)
         *min_height = 0.0;
@@ -333,7 +348,7 @@ clutter_box_real_allocate (ClutterActor           *actor,
   klass = CLUTTER_ACTOR_CLASS (clutter_box_parent_class);
   klass->allocate (actor, allocation, flags);
 
-  if (priv->children == NULL)
+  if (priv->children == NULL || priv->manager == NULL)
     return;
 
   clutter_layout_manager_allocate (priv->manager,
@@ -349,7 +364,8 @@ clutter_box_destroy (ClutterActor *actor)
   /* destroy all our children */
   g_list_foreach (priv->children, (GFunc) clutter_actor_destroy, NULL);
 
-  CLUTTER_ACTOR_CLASS (clutter_box_parent_class)->destroy (actor);
+  if (CLUTTER_ACTOR_CLASS (clutter_box_parent_class)->destroy)
+    CLUTTER_ACTOR_CLASS (clutter_box_parent_class)->destroy (actor);
 }
 
 static inline void
@@ -668,48 +684,21 @@ clutter_box_packv (ClutterBox          *box,
     }
 }
 
-/**
- * clutter_box_pack:
- * @box: a #ClutterBox
- * @actor: a #ClutterActor
- * @first_property: the name of the first property to set, or %NULL
- * @Varargs: a list of property name and value pairs, terminated by %NULL
- *
- * Adds @actor to @box and sets layout properties at the same time,
- * if the #ClutterLayoutManager used by @box has them
- *
- * This function is a wrapper around clutter_container_add_actor()
- * and clutter_layout_manager_child_set()
- *
- * Language bindings should use the vector-based clutter_box_addv()
- * variant instead
- *
- * Since: 1.2
- */
-void
-clutter_box_pack (ClutterBox   *box,
-                  ClutterActor *actor,
-                  const gchar  *first_property,
-                  ...)
+static inline void
+clutter_box_set_property_valist (ClutterBox   *box,
+                                 ClutterActor *actor,
+                                 const gchar  *first_property,
+                                 va_list       var_args)
 {
-  ClutterBoxPrivate *priv;
-  ClutterContainer *container;
+  ClutterContainer *container = CLUTTER_CONTAINER (box);
+  ClutterBoxPrivate *priv = box->priv;
   ClutterLayoutMeta *meta;
   GObjectClass *klass;
   const gchar *pname;
-  va_list var_args;
 
-  g_return_if_fail (CLUTTER_IS_BOX (box));
-  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
-
-  container = CLUTTER_CONTAINER (box);
-  clutter_container_add_actor (container, actor);
-
-  if (first_property == NULL || *first_property == '\0')
+  if (priv->manager == NULL)
     return;
 
-  priv = box->priv;
-
   meta = clutter_layout_manager_get_child_meta (priv->manager,
                                                 container,
                                                 actor);
@@ -719,8 +708,6 @@ clutter_box_pack (ClutterBox   *box,
 
   klass = G_OBJECT_GET_CLASS (meta);
 
-  va_start (var_args, first_property);
-
   pname = first_property;
   while (pname)
     {
@@ -768,7 +755,183 @@ clutter_box_pack (ClutterBox   *box,
 
       pname = va_arg (var_args, gchar*);
     }
+}
 
+/**
+ * clutter_box_pack:
+ * @box: a #ClutterBox
+ * @actor: a #ClutterActor
+ * @first_property: the name of the first property to set, or %NULL
+ * @Varargs: a list of property name and value pairs, terminated by %NULL
+ *
+ * Adds @actor to @box and sets layout properties at the same time,
+ * if the #ClutterLayoutManager used by @box has them
+ *
+ * This function is a wrapper around clutter_container_add_actor()
+ * and clutter_layout_manager_child_set()
+ *
+ * Language bindings should use the vector-based clutter_box_addv()
+ * variant instead
+ *
+ * Since: 1.2
+ */
+void
+clutter_box_pack (ClutterBox   *box,
+                  ClutterActor *actor,
+                  const gchar  *first_property,
+                  ...)
+{
+  va_list var_args;
+
+  g_return_if_fail (CLUTTER_IS_BOX (box));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+
+  clutter_container_add_actor (CLUTTER_CONTAINER (box), actor);
+
+  if (first_property == NULL || *first_property == '\0')
+    return;
+
+  va_start (var_args, first_property);
+  clutter_box_set_property_valist (box, actor, first_property, var_args);
+  va_end (var_args);
+}
+
+/**
+ * clutter_box_pack_after:
+ * @box: a #ClutterBox
+ * @actor: a #ClutterActor
+ * @sibling: (allow none): a #ClutterActor or %NULL
+ * @first_property: the name of the first property to set, or %NULL
+ * @Varargs: a list of property name and value pairs, terminated by %NULL
+ *
+ * Adds @actor to @box, placing it after @sibling, and sets layout
+ * properties at the same time, if the #ClutterLayoutManager used by
+ * @box supports them
+ *
+ * If @sibling is %NULL then @actor is placed at the end of the
+ * list of children, to be allocated and painted after every other child
+ *
+ * This function is a wrapper around clutter_container_add_actor(),
+ * clutter_container_raise_child() and clutter_layout_manager_child_set()
+ *
+ * Since: 1.2
+ */
+void
+clutter_box_pack_after (ClutterBox   *box,
+                        ClutterActor *actor,
+                        ClutterActor *sibling,
+                        const gchar  *first_property,
+                        ...)
+{
+  va_list var_args;
+
+  g_return_if_fail (CLUTTER_IS_BOX (box));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+  g_return_if_fail (sibling == NULL || CLUTTER_IS_ACTOR (sibling));
+
+  clutter_container_add_actor (CLUTTER_CONTAINER (box), actor);
+  clutter_container_raise_child (CLUTTER_CONTAINER (box), actor, sibling);
+
+  if (first_property == NULL || *first_property == '\0')
+    return;
+
+  va_start (var_args, first_property);
+  clutter_box_set_property_valist (box, actor, first_property, var_args);
+  va_end (var_args);
+}
+
+/**
+ * clutter_box_pack_before:
+ * @box: a #ClutterBox
+ * @actor: a #ClutterActor
+ * @sibling: (allow none): a #ClutterActor or %NULL
+ * @first_property: the name of the first property to set, or %NULL
+ * @Varargs: a list of property name and value pairs, terminated by %NULL
+ *
+ * Adds @actor to @box, placing it before @sibling, and sets layout
+ * properties at the same time, if the #ClutterLayoutManager used by
+ * @box supports them
+ *
+ * If @sibling is %NULL then @actor is placed at the beginning of the
+ * list of children, to be allocated and painted below every other child
+ *
+ * This function is a wrapper around clutter_container_add_actor(),
+ * clutter_container_lower_child() and clutter_layout_manager_child_set()
+ *
+ * Since: 1.2
+ */
+void
+clutter_box_pack_before (ClutterBox   *box,
+                         ClutterActor *actor,
+                         ClutterActor *sibling,
+                         const gchar  *first_property,
+                         ...)
+{
+  va_list var_args;
+
+  g_return_if_fail (CLUTTER_IS_BOX (box));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+  g_return_if_fail (sibling == NULL || CLUTTER_IS_ACTOR (sibling));
+
+  clutter_container_add_actor (CLUTTER_CONTAINER (box), actor);
+  clutter_container_lower_child (CLUTTER_CONTAINER (box), actor, sibling);
+
+  if (first_property == NULL || *first_property == '\0')
+    return;
+
+  va_start (var_args, first_property);
+  clutter_box_set_property_valist (box, actor, first_property, var_args);
+  va_end (var_args);
+}
+
+/**
+ * clutter_box_pack_at:
+ * @box: a #ClutterBox
+ * @actor: a #ClutterActor
+ * @position: the position to insert the @actor at
+ * @first_property: the name of the first property to set, or %NULL
+ * @Varargs: a list of property name and value pairs, terminated by %NULL
+ *
+ * Adds @actor to @box, placing it at @position, and sets layout
+ * properties at the same time, if the #ClutterLayoutManager used by
+ * @box supports them
+ *
+ * If @position is a negative number, or is larger than the number of
+ * children of @box, the new child is added at the end of the list of
+ * children
+ *
+ * Since: 1.2
+ */
+void
+clutter_box_pack_at (ClutterBox   *box,
+                     ClutterActor *actor,
+                     gint          position,
+                     const gchar  *first_property,
+                     ...)
+{
+  ClutterBoxPrivate *priv;
+  va_list var_args;
+
+  g_return_if_fail (CLUTTER_IS_BOX (box));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+
+  priv = box->priv;
+
+  /* this is really clutter_box_add() with a different insert() */
+  priv->children = g_list_insert (priv->children,
+                                  actor,
+                                  position);
+
+  clutter_actor_set_parent (actor, CLUTTER_ACTOR (box));
+  clutter_actor_queue_relayout (actor);
+
+  g_signal_emit_by_name (box, "actor-added", actor);
+
+  if (first_property == NULL || *first_property == '\0')
+    return;
+
+  va_start (var_args, first_property);
+  clutter_box_set_property_valist (box, actor, first_property, var_args);
   va_end (var_args);
 }
 
