@@ -18,6 +18,7 @@ clutter_actor_flags_get_type (void)
         { CLUTTER_ACTOR_REALIZED, "CLUTTER_ACTOR_REALIZED", "realized" },
         { CLUTTER_ACTOR_REACTIVE, "CLUTTER_ACTOR_REACTIVE", "reactive" },
         { CLUTTER_ACTOR_VISIBLE, "CLUTTER_ACTOR_VISIBLE", "visible" },
+        { CLUTTER_ACTOR_NO_LAYOUT, "CLUTTER_ACTOR_NO_LAYOUT", "no-layout" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
@@ -53,6 +54,32 @@ clutter_allocation_flags_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "./clutter-animator.h" */
+#include "./clutter-animator.h"
+
+GType
+clutter_interpolation_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_INTERPOLATION_LINEAR, "CLUTTER_INTERPOLATION_LINEAR", "linear" },
+        { CLUTTER_INTERPOLATION_CUBIC, "CLUTTER_INTERPOLATION_CUBIC", "cubic" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterInterpolation"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "./clutter-bin-layout.h" */
 #include "./clutter-bin-layout.h"
 
@@ -252,30 +279,6 @@ clutter_stage_state_get_type (void)
 
   return g_enum_type_id__volatile;
 }
-GType
-clutter_input_device_type_get_type (void)
-{
-  static volatile gsize g_enum_type_id__volatile = 0;
-
-  if (g_once_init_enter (&g_enum_type_id__volatile))
-    {
-      static const GEnumValue values[] = {
-        { CLUTTER_POINTER_DEVICE, "CLUTTER_POINTER_DEVICE", "pointer-device" },
-        { CLUTTER_KEYBOARD_DEVICE, "CLUTTER_KEYBOARD_DEVICE", "keyboard-device" },
-        { CLUTTER_EXTENSION_DEVICE, "CLUTTER_EXTENSION_DEVICE", "extension-device" },
-        { CLUTTER_N_DEVICE_TYPES, "CLUTTER_N_DEVICE_TYPES", "n-device-types" },
-        { 0, NULL, NULL }
-      };
-      GType g_enum_type_id;
-
-      g_enum_type_id =
-        g_enum_register_static (g_intern_static_string ("ClutterInputDeviceType"), values);
-
-      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
-    }
-
-  return g_enum_type_id__volatile;
-}
 
 /* enumerations from "./clutter-feature.h" */
 #include "./clutter-feature.h"
@@ -298,6 +301,7 @@ clutter_feature_flags_get_type (void)
         { CLUTTER_FEATURE_SHADERS_GLSL, "CLUTTER_FEATURE_SHADERS_GLSL", "shaders-glsl" },
         { CLUTTER_FEATURE_OFFSCREEN, "CLUTTER_FEATURE_OFFSCREEN", "offscreen" },
         { CLUTTER_FEATURE_STAGE_MULTIPLE, "CLUTTER_FEATURE_STAGE_MULTIPLE", "stage-multiple" },
+        { CLUTTER_FEATURE_SWAP_EVENTS, "CLUTTER_FEATURE_SWAP_EVENTS", "swap-events" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
@@ -337,6 +341,34 @@ clutter_flow_orientation_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "./clutter-input-device.h" */
+#include "./clutter-input-device.h"
+
+GType
+clutter_input_device_type_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_POINTER_DEVICE, "CLUTTER_POINTER_DEVICE", "pointer-device" },
+        { CLUTTER_KEYBOARD_DEVICE, "CLUTTER_KEYBOARD_DEVICE", "keyboard-device" },
+        { CLUTTER_EXTENSION_DEVICE, "CLUTTER_EXTENSION_DEVICE", "extension-device" },
+        { CLUTTER_N_DEVICE_TYPES, "CLUTTER_N_DEVICE_TYPES", "n-device-types" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterInputDeviceType"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "./clutter-main.h" */
 #include "./clutter-main.h"
 
@@ -753,6 +785,29 @@ clutter_font_flags_get_type (void)
 
   return g_enum_type_id__volatile;
 }
+GType
+clutter_text_direction_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_TEXT_DIRECTION_DEFAULT, "CLUTTER_TEXT_DIRECTION_DEFAULT", "default" },
+        { CLUTTER_TEXT_DIRECTION_LTR, "CLUTTER_TEXT_DIRECTION_LTR", "ltr" },
+        { CLUTTER_TEXT_DIRECTION_RTL, "CLUTTER_TEXT_DIRECTION_RTL", "rtl" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterTextDirection"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
 
 /* enumerations from "./clutter-units.h" */
 #include "./clutter-units.h"
