@@ -34,6 +34,7 @@
 #include "clutter-alpha.h"
 #include "clutter-animatable.h"
 #include "clutter-animation.h"
+#include "clutter-animator.h"
 #include "clutter-backend.h"
 #include "clutter-behaviour-depth.h"
 #include "clutter-behaviour-ellipse.h"
@@ -51,12 +52,14 @@
 #include "clutter-clone.h"
 #include "clutter-color.h"
 #include "clutter-container.h"
+#include "clutter-device-manager.h"
 #include "clutter-event.h"
 #include "clutter-feature.h"
 #include "clutter-fixed-layout.h"
 #include "clutter-flow-layout.h"
 #include "clutter-frame-source.h"
 #include "clutter-group.h"
+#include "clutter-input-device.h"
 #include "clutter-interval.h"
 #include "clutter-keysyms.h" 
 #include "clutter-layout-manager.h"
