@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors:
  *  Neil Roberts   <neil@linux.intel.com>
@@ -268,8 +268,6 @@ _cogl_atlas_texture_free (CoglAtlasTexture *atlas_tex)
   _cogl_atlas_texture_remove_from_atlas (atlas_tex);
 
   cogl_handle_unref (atlas_tex->sub_texture);
-
-  g_free (atlas_tex);
 }
 
 static int
@@ -310,7 +308,7 @@ _cogl_atlas_texture_transform_coords_to_gl (CoglTexture *tex,
   _cogl_texture_transform_coords_to_gl (atlas_tex->sub_texture, s, t);
 }
 
-static CoglTransformResult
+static gboolean
 _cogl_atlas_texture_transform_quad_coords_to_gl (CoglTexture *tex,
                                                  float *coords)
 {
