@@ -107,7 +107,7 @@ cogl_shader_compile (CoglHandle handle)
   glCompileShader (shader->gl_handle);
 }
 
-gchar *
+char *
 cogl_shader_get_info_log (CoglHandle handle)
 {
   CoglShader *shader;
