@@ -47,6 +47,33 @@
 
 #include "cogl-gles2-wrapper.h"
 
+void
+_cogl_texture_driver_gen (GLenum   gl_target,
+                          GLsizei  n,
+                          GLuint  *textures)
+{
+  unsigned int i;
+
+  GE (glGenTextures (n, textures));
+
+  for (i = 0; i < n; i++)
+    {
+      GE (glBindTexture (gl_target, textures[i]));
+
+      switch (gl_target)
+        {
+      case GL_TEXTURE_2D:
+        /* GL_TEXTURE_MAG_FILTER defaults to GL_LINEAR, no need to set it */
+        GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR) );
+        break;
+
+      default:
+        g_assert_not_reached();
+        }
+
+    }
+}
+
 
 void
 _cogl_texture_driver_bind (GLenum gl_target,
@@ -71,7 +98,8 @@ _cogl_texture_driver_prep_gl_for_pixels_download (int pixels_rowstride,
 }
 
 void
-_cogl_texture_driver_upload_subregion_to_gl (CoglTexture *tex,
+_cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
+                                             GLuint       gl_handle,
                                              int          src_x,
                                              int          src_y,
                                              int          dst_x,
@@ -80,8 +108,7 @@ _cogl_texture_driver_upload_subregion_to_gl (CoglTexture *tex,
                                              int          height,
                                              CoglBitmap  *source_bmp,
 				             GLuint       source_gl_format,
-				             GLuint       source_gl_type,
-                                             GLuint       gl_handle)
+				             GLuint       source_gl_type)
 {
   int bpp = _cogl_get_format_bpp (source_bmp->format);
   CoglBitmap slice_bmp;
@@ -94,11 +121,11 @@ _cogl_texture_driver_upload_subregion_to_gl (CoglTexture *tex,
    * rowstride = bpp * width and the texture image is not sliced */
 
   /* Setup temp bitmap for slice subregion */
-  slice_bmp.format = tex->bitmap.format;
+  slice_bmp.format = source_bmp->format;
   slice_bmp.width  = width;
   slice_bmp.height = height;
   slice_bmp.rowstride = bpp * slice_bmp.width;
-  slice_bmp.data = (guchar*) g_malloc (slice_bmp.rowstride * slice_bmp.height);
+  slice_bmp.data = g_malloc (slice_bmp.rowstride * slice_bmp.height);
 
   /* Setup gl alignment to match rowstride and top-left corner */
   _cogl_texture_driver_prep_gl_for_pixels_upload (slice_bmp.rowstride,
@@ -113,11 +140,11 @@ _cogl_texture_driver_upload_subregion_to_gl (CoglTexture *tex,
                                slice_bmp.width,
                                slice_bmp.height);
 
-  /* Upload new image data */
-  GE( _cogl_texture_driver_bind (tex->gl_target,
-                                 gl_handle, tex->gl_intformat) );
+  /* We don't need to use _cogl_texture_driver_bind here because we're
+     not using the bound texture to render yet */
+  GE( glBindTexture (gl_target, gl_handle) );
 
-  GE( glTexSubImage2D (tex->gl_target, 0,
+  GE( glTexSubImage2D (gl_target, 0,
                        dst_x, dst_y,
                        width, height,
                        source_gl_format,
@@ -128,6 +155,53 @@ _cogl_texture_driver_upload_subregion_to_gl (CoglTexture *tex,
   g_free (slice_bmp.data);
 }
 
+void
+_cogl_texture_driver_upload_to_gl (GLenum       gl_target,
+                                   GLuint       gl_handle,
+                                   CoglBitmap  *source_bmp,
+                                   GLint        internal_gl_format,
+                                   GLuint       source_gl_format,
+                                   GLuint       source_gl_type)
+{
+  int bpp = _cogl_get_format_bpp (source_bmp->format);
+  CoglBitmap bmp = *source_bmp;
+  gboolean bmp_owner = FALSE;
+
+  /* If the rowstride can't be specified with just GL_ALIGNMENT alone
+     then we need to copy the bitmap because there is no GL_ROW_LENGTH */
+  if (source_bmp->rowstride / bpp != source_bmp->width)
+    {
+      bmp.rowstride = bpp * bmp.width;
+      bmp.data = g_malloc (bmp.rowstride * bmp.height);
+      bmp_owner = TRUE;
+
+      _cogl_bitmap_copy_subregion (source_bmp,
+                                   &bmp,
+                                   0, 0, 0, 0,
+                                   bmp.width,
+                                   bmp.height);
+    }
+
+  /* Setup gl alignment to match rowstride and top-left corner */
+  _cogl_texture_driver_prep_gl_for_pixels_upload (bmp.rowstride,
+                                                  bpp);
+
+  /* We don't need to use _cogl_texture_driver_bind here because we're
+     not using the bound texture to render yet */
+  GE( glBindTexture (gl_target, gl_handle) );
+
+  GE( glTexImage2D (gl_target, 0,
+                    internal_gl_format,
+                    bmp.width, bmp.height,
+                    0,
+                    source_gl_format,
+                    source_gl_type,
+                    bmp.data) );
+
+  if (bmp_owner)
+    g_free (bmp.data);
+}
+
 /* NB: GLES doesn't support glGetTexImage2D, so cogl-texture will instead
  * fallback to a generic render + readpixels approach to downloading
  * texture data. (See _cogl_texture_draw_and_read() ) */
